export interface IKeyParams {
    type: 'string' | 'number';
    name: string;
}
export interface IIndexParams {
    hash: string | IKeyParams;
    sort?: string | IKeyParams;
}
export interface IIndex {
    hash: IKeyParams;
    sort?: IKeyParams;
}
export interface IndexSchemaParams {
    pk: IIndexParams;
    s1k: IIndexParams;
    s2k: IIndexParams;
    s3k: IIndexParams;
    s4k: IIndexParams;
    s5k: IIndexParams;
    s6k: IIndexParams;
    s7k: IIndexParams;
    s8k: IIndexParams;
    s9k: IIndexParams;
    s10k: IIndexParams;
    s11k: IIndexParams;
}
export declare type IndexName = keyof IndexSchemaParams;
export declare type IndexSchema = Record<IndexName, IIndex>;
export interface IStorageParams {
    tableName: string;
    indices?: IndexName[];
}
export interface IStorage {
}
export declare class Storage implements IStorage {
    readonly indexSchema: Partial<IndexSchema>;
    readonly tableName: string;
    constructor({ tableName, indices }: IStorageParams);
    getKeyName(indexName: IndexName, keyType: 'hash' | 'sort'): string | undefined;
    getPageToken(indexName: IndexName, lastKey: Record<string, string | number>): {
        hash: string | number;
    } & {
        sort: string | number;
    };
}
export declare function fromIndexString(index: string): {
    [x: string]: {
        hash: {
            type: string;
            name: string;
        };
        sort: {
            type: string;
            name: string;
        };
    };
};
export declare function fromKeyString(name: string): {
    type: string;
    name: string;
};
